<?php
if (!defined('ABSPATH')) die('No direct access.');

if (class_exists('MPSUM_Auto_Rollback')) return;

/**
 * Class MPSUM_Auto_Rollback rolls back updates if plugins or themes fail
 */
class MPSUM_Auto_Rollback {

	/**
	 * Returns singleton instance of this class
	 *
	 * @return object MPSUM_Auto_Rollback Singleton Instance
	 */
	public static function get_instance() {
		static $instance = null;
		if (null === $instance) {
			$instance = new self();
		}
		return $instance;
	}

	/**
	 * Adds necessary filters and actions
	 */
	private function __construct() {
		add_action('pre_auto_update', array($this, 'get_active_plugins'));
		add_action('automatic_updates_complete', array($this, 'check_automatic_updates'), 10, 1);
		add_action('automatic_updates_complete', array($this, 'check_active_plugins'), 100, 1);
	}

	/**
	 * Get active plugins before update
	 *
	 * @since 8.0.1
	 * @access public
	 * @return void
	 */
	public function get_active_plugins() {
		delete_site_option('eum_active_pre_restore_plugins_multisite');
		delete_site_option('eum_active_pre_restore_plugins');
		if (is_multisite()) {
			$active_plugins_multisite = MPSUM_Utils::get_instance()->get_network_active_plugins();
			update_site_option('eum_active_pre_restore_plugins_multisite', $active_plugins_multisite);
		} else {
			$active_plugins_single_site = MPSUM_Utils::get_instance()->get_single_site_active_plugins();
			update_site_option('eum_active_pre_restore_plugins', $active_plugins_single_site);
		}
	}

	/**
	 * Check to see if plugins are activated - Send an email otherwise
	 *
	 * @since 8.0.1
	 * @access public
	 * @param array $update_results Update results
	 * @return void
	 */
	public function check_active_plugins($update_results) {
		$active_plugins_pre = get_site_option('eum_active_pre_restore_plugins', array());
		$active_plugins_pre_multisite = get_site_option('eum_active_pre_restore_plugins_multisite', array());
		$message = '';
		$maybe_messsage_multisite = __('A previously network-activated plugin has been deactivated, possibly due to an update error. Please check your site for any plugins that may have been deactivated in error.', 'stops-core-theme-and-plugin-updates') . "\r\n\r\n";
		$maybe_messsage_multisite .= __('Here are the plugins that have been deactivated:', 'stops-core-theme-and-plugin-updates') . "\r\n\r\n";
		$maybe_message_single_site = __('A previously active plugin has been deactivated, possibly due to an update error. Please check your site for any plugins that may have been deactivated in error.', 'stops-core-theme-and-plugin-updates') . "\r\n\r\n";
		$maybe_message_single_site .= __('Here are the plugins that have been deactivated:', 'stops-core-theme-and-plugin-updates') . "\r\n\r\n";
		
		foreach ($update_results as $type => $results) {
			if ('plugin' === $type) {
				foreach ($results as $plugin) {
					if (!isset($plugin->item->plugin)) continue;
					$plugin_slug = $plugin->item->plugin;
					$name = (isset($plugin->name) && !empty($plugin->name)) ? $plugin->name : $plugin->item->slug;
					$version = isset($plugin->item->new_version) ? $plugin->item->new_version : '0.00';
					
					if (!is_multisite()) {
						foreach ($active_plugins_pre as $saved_plugin_slug) {
							if (!is_plugin_active($saved_plugin_slug)) {
								// Attempt to reactivate
								$message .= sprintf(__('Plugin: %s Version: %s', 'stops-core-theme-and-plugin-updates'), esc_html($name), esc_html($version)). "\r\n";
								$message .= __('We are attempting to reactivate the plugin.', 'stops-core-theme-and-plugin-updates') . "\r\n";
								$maybe_success = activate_plugin($saved_plugin_slug, '', false, true);
								if (null === $maybe_success && is_plugin_active($saved_plugin_slug)) {
									$message .= __('Re-activation has been successful.', 'stops-core-theme-and-plugin-updates') . "\r\n\r\n";
								} else {
									$message .= __('Re-activation was not successful.', 'stops-core-theme-and-plugin-updates') . "\r\n\r\n";
								}
							}
						}
					}
					if (is_multisite()) {
						foreach ($active_plugins_pre_multisite as $multisite_saved_plugin_slug) {
							if (!is_plugin_active_for_network($multisite_saved_plugin_slug)) {
								// Attempt to reactivate
								$message .= sprintf(__('Plugin: %s Version: %s', 'stops-core-theme-and-plugin-updates'), esc_html($name), esc_html($version)). "\r\n";
								$message .= __('We are attempting to network activate the plugin.', 'stops-core-theme-and-plugin-updates') . "\r\n";
								$maybe_success = activate_plugin($multisite_saved_plugin_slug, '', true, true );
								if (null === $maybe_success && is_plugin_active_for_network($multisite_saved_plugin_slug)) {
									$message .= __('Re-activation has been successful.', 'stops-core-theme-and-plugin-updates') . "\r\n\r\n";
								} else {
									$message .= __('Re-activation was not successful.', 'stops-core-theme-and-plugin-updates') . "\r\n\r\n";
								}
							}
						}
					}
				}
			}
		}
		if (!empty($message)) {
			if (is_multisite()) {
				$message = $maybe_messsage_multisite . $message;
			} else {
				$message = $maybe_message_single_site . $message;
			}
			/**
			 * Change the subject of the deactivation notification email.
			 *
			 * @since 8.0.1
			 *
			 * @param string Email Subject
			 * @param string URL of site or network
			 */
			$subject = apply_filters('eum_deactivation_subject', sprintf(__('Please check your site immediately as some plugins were not reactivated after an update: %s', 'stops-core-theme-and-plugin-updates'), esc_url(network_site_url())));
			
			// Get site name
			$sitename = is_multisite() ? get_site_option('site_name') : get_option('blogname');

			// Get Send E-mail
			$sender_email = is_multisite() ? get_site_option('admin_email') : get_option('admin_email');

			$to = MPSUM_Utils::get_instance()->get_emails();

			// Set headers
			$headers = array();
			$headers[] = sprintf('From: %s <%s>', esc_html($sitename), $sender_email);
			
			/**
			 * Allow others to provide an action prior to emailing.
			 *
			 * Allow others to provide an action prior to emailing.
			 *
			 * @since 8.0.1
			 *
			 * @param mixed  $to Array or string of emails to send.
			 * @param string $subject        Subject of the email
			 * @param string $message        Message of the email
			 * @param array  $headers        Headers to be sent via email
			 * @param array  $update_results Results from the automatic update proccess
			 */
			do_action('eum_deacivation_before_send', $to, $subject, $message, $headers, $update_results);

			/**
			 * Allow others to override email sending.
			 *
			 * Allow others to override email sending.
			 *
			 * @since 8.0.1
			 *
			 * @param bool   Whether to send the email or not
			 * @param mixed  $to             Array or string of emails to send.
			 * @param string $subject        Subject of the email
			 * @param string $message        Message of the email
			 * @param array  $headers        Headers to be sent via email
			 * @param array  $update_results Results from the automatic update proccess
			 *
			 * @return bool Whether to send the email or not
			 */
			$allow_send_email = apply_filters('eum_deacivation_before_send_email', true, $to, $subject, $message, $headers, $update_results);

			if ($allow_send_email) {
				// Send email
				wp_mail($to, $subject, $message, $headers);
			}

			/**
			 * Allow others to provide an action after emailing.
			 *
			 * Allow others to provide an action after emailing.
			 *
			 * @since 8.0.1
			 *
			 * @param bool   $allow_send_email Boolean to determine if emails have been sent (default true)
			 * @param mixed  $to               Array or string of emails to send.
			 * @param string $subject          Subject of the email
			 * @param string $message          Message of the email
			 * @param array  $headers          Headers to be sent via email
			 * @param array  $update_resultst  Update results
			 */
			do_action('eum_deactivation_after_send', $allow_send_email, $to, $subject, $message, $headers, $update_results);
		}

		
		
	}

	/**
	 * Check automatic updates for failures
	 *
	 * @since 8.0.1
	 * @access public
	 * @param array $update_results Update results
	 * @return void
	 */
	public function check_automatic_updates($update_results) {
		if (empty($update_results)) return;
		$white_screen_check = $this->perform_whitescreen_check();
		if (true !== $white_screen_check) {
			if (isset($white_screen_check['code']) && 'loopback_request_failed' === $white_screen_check['code']) {
				return;
			} elseif (is_array($white_screen_check)) {
				$this->perform_deactivation($update_results);
			}
		}
	}

	/**
	 * Check the site for WSOD
	 *
	 * @since 8.0.1
	 * @access private
	 * @return mixed true if everything okay, array if not
	 */
	private function perform_whitescreen_check() {
		
		$scrape_key = md5(rand());
		$transient = 'scrape_key_' . $scrape_key;
		$scrape_nonce = strval(rand());
		set_transient($transient, $scrape_nonce, 60); // It shouldn't take more than 60 seconds to make one loopback request.
		
		$needle_start = "###### wp_scraping_result_start:$scrape_key ######";
		$needle_end = "###### wp_scraping_result_end:$scrape_key ######";
		$scrape_params = array(
			'wp_scrape_key' => $scrape_key,
			'wp_scrape_nonce' => $scrape_nonce,
		);
		$headers = array(
			'Cache-Control' => 'no-cache',
		);

		// Make sure PHP process doesn't die before loopback requests complete.
		@set_time_limit(300);

		// Time to wait for loopback requests to finish.
		$timeout = 30;

		// Setup some failure variables
		$loopback_request_failure = array(
			'code' => 'loopback_request_failed',
			'message' => __('Unable to communicate back with site to check for fatal errors. You will want to update your assets manually.', 'stops-core-theme-and-plugin-updates'),
		);
		$json_parse_failure = array(
			'code' => 'json_parse_error',
		);

		// Set and perform loopback
		$url = home_url('/');
		$url = add_query_arg($scrape_params, $url);
		$r = wp_remote_get($url, compact('headers', 'timeout'));
		$body = wp_remote_retrieve_body($r);
		$scrape_result_position = strpos($body, $needle_start);

		// Check for scrape variables
		$result = null;
		if (false === $scrape_result_position) {
			$result = $loopback_request_failure;
		} else {
			$error_output = substr($body, $scrape_result_position + strlen($needle_start));
			$error_output = substr($error_output, 0, strpos($error_output, $needle_end));
			$result = json_decode(trim($error_output), true);
			if (empty($result)) {
				$result = $json_parse_failure;
			}
		}
		delete_transient($transient);

		return $result;
	}

	/**
	 * Perform a notice and deactivation of plugins on WSOD error
	 *
	 * @since 8.0.1
	 * @access private
	 * @param array $update_results
	 */
	private function perform_deactivation($update_results) {
		
		// Detail in email the update type
		$updates_message = '';
		foreach ($update_results as $type => $results) {
			switch ($type) {
				case 'core':
					$core = $results[0];
					$name = $core->name;
					$updates_message .= __('== WordPress Core ==', 'stops-core-theme-and-plugin-updates') . "\r\n\r\n";
					$updates_message .= $name . "\r\n";
					break;
				case 'plugin':
					if (!empty($results)) {
						$updates_message .= "\r\n" . __('== WordPress Plugins ==', 'stops-core-theme-and-plugin-updates') . "\r\n\r\n";
						foreach ($results as $plugin) {
							$name = (isset($plugin->name) && !empty($plugin->name)) ? $plugin->name : $plugin->item->slug;
							$version = isset($plugin->item->new_version) ? $plugin->item->new_version : '0.00';
							$updates_message .= sprintf(__('%s: Version %s', 'stops-core-theme-and-plugin-updates'), esc_html($name), esc_html($version)) . "\r\n";
							deactivate_plugins($plugin->item->plugin);
							$updates_message .= sprintf(__('%s: Deactivated as a Precaution', 'stops-core-theme-and-plugin-updates'), esc_html($name)) . "\r\n\r\n";
						}
					}
					break;
				case 'theme':
					if (!empty($results)) {
						$updates_message .= "\r\n" . __('== WordPress Themes ==', 'stops-core-theme-and-plugin-updates') . "\r\n\r\n";
						foreach ($results as $theme) {
							$name = $theme->name;
							$version = $theme->item->new_version;
							$updates_message .= sprintf(__('%s: Version %s', 'stops-core-theme-and-plugin-updates'), esc_html($name), esc_html($version)) . "\r\n";
						}
					}
					break;
			}
		}

		// Get site name
		$sitename = is_multisite() ? get_site_option('site_name') : get_option('blogname');

		// Get Send E-mail
		$sender_email = is_multisite() ? get_site_option('admin_email') : get_option('admin_email');

		// Set headers
		$headers = array();
		$headers[] = sprintf('From: %s <%s>', esc_html($sitename), $sender_email);

		/**
		 * Change the subject of the rollback notification email.
		 *
		 * @since 8.0.1
		 *
		 * @param string Email Subject
		 * @param string URL of site or network
		 */
		$subject = apply_filters('eum_rollback_subject', sprintf(__('Please check your site immediately: %s', 'stops-core-theme-and-plugin-updates'), esc_url(network_site_url())));
		$white_screen_message = __('Please view your site immediately. One of your plugins caused a fatal PHP error. We have deactivated problem plugins during this update. You may want to attempt to re-activate them.', 'stops-core-theme-and-plugin-updates') . "\r\n";
		$to = MPSUM_Utils::get_instance()->get_emails();

		/**
		 * Allow others to provide an action prior to emailing.
		 *
		 * Allow others to provide an action prior to emailing.
		 *
		 * @since 8.0.1
		 *
		 * @param mixed  $to Array or string of emails to send.
		 * @param string $subject        Subject of the email
		 * @param string $message        Message of the email
		 * @param array  $headers        Headers to be sent via email
		 * @param array  $update_results Results from the automatic update proccess
		 */
		do_action('eum_rollback_before_send', $to, $subject, $white_screen_message . $updates_message, $headers, $update_results);

		/**
		 * Allow others to override email sending.
		 *
		 * Allow others to override email sending.
		 *
		 * @since 8.0.1
		 *
		 * @param bool   Whether to send the email or not
		 * @param mixed  $to             Array or string of emails to send.
		 * @param string $subject        Subject of the email
		 * @param string $message        Message of the email
		 * @param array  $headers        Headers to be sent via email
		 * @param array  $update_results Results from the automatic update proccess
		 *
		 * @return bool Whether to send the email or not
		 */
		$allow_send_email = apply_filters('eum_rollback_before_send_email', true, $to, $subject, $white_screen_message . $updates_message, $headers, $update_results);

		if ($allow_send_email) {
			// Send email
			wp_mail($to, $subject, $white_screen_message . $updates_message, $headers);
		}

		/**
		 * Allow others to provide an action after emailing.
		 *
		 * Allow others to provide an action after emailing.
		 *
		 * @since 8.0.1
		 *
		 * @param bool   $allow_send_email Boolean to determine if emails have been sent (default true)
		 * @param mixed  $to               Array or string of emails to send.
		 * @param string $subject          Subject of the email
		 * @param string $message          Message of the email
		 * @param array  $headers          Headers to be sent via email
		 * @param array  $update_resultst  Update results
		 */
		do_action('eum_rollback_after_send', $allow_send_email, $to, $subject, $white_screen_message . $updates_message, $headers, $update_results);
	}
}
