<?php
if (!defined('ABSPATH')) die('No direct access.');

if (class_exists('MPSUM_External_Logs')) return;

/**
 * Class MPSUM_External_Logs
 */
class MPSUM_External_Logs extends MPSUM_Logs {

	protected $logger = null;

	/**
	 * MPSUM_External_Logs constructor. Initiates and adds necessary hooks
	 */
	protected function __construct() {
		parent::__construct();
		add_action('eum_logs', array($this, 'display_settings'));
		add_action('automatic_updates_complete', array($this, 'automatic_updates'));
		add_action('upgrader_process_complete', array($this, 'manual_updates'), 10, 2);
		$this->setup_loggers();
	}

	/**
	 * Returns instance of singleton pattern
	 *
	 * @return MPSUM_External_Logs
	 */
	public static function get_instance() {
		static $instance = null;
		if (null === $instance) {
			$instance = new self();
		}
		return $instance;
	}

	/**
	 * Setup External logger(s)
	 */
	public function setup_loggers() {
		$logger = $this->get_logger();
		$loggers = $this->eum_loggers();

		if (!empty($loggers)) {
			foreach ($loggers as $_logger) {
				$logger->add_logger($_logger);
			}
		}
	}

	/**
	 * Gets logger instance
	 *
	 * @return MPSUM_Logger
	 */
	public function get_logger() {
		static $_instance = null;
		if (null === $_instance) {
			$_instance = new MPSUM_Logger();
		}
		return $_instance;
	}

	/**
	 * Returns list of EUM loggers instances
	 * Apply filter eum_loggers
	 *
	 * @return array
	 */
	public function eum_loggers() {
		$loggers = array();
		$options_keys = array();
		$loggers_classes = $this->get_loggers_classes();

		$options = MPSUM_Updates_Manager::get_options('logs');
		$logger_additional_options = isset($options['logger_additional_options']) ? $options['logger_additional_options'] : '';

		// create loggers classes instances.
		if (!empty($loggers_classes)) {

			foreach ($loggers_classes as $logger_class => $source) {

				$logger_id = strtolower($logger_class);
				$logger = new $logger_class();

				$logger_options = $logger->get_options_list();

				if (!empty($logger_options)) {
					foreach (array_keys($logger_options) as $option_name) {
						if (array_key_exists($option_name, $options_keys)) {
							$options_keys[$option_name]++;
						} else {
							$options_keys[$option_name] = 0;
						}

						$option_value = isset($logger_additional_options[$option_name]) ? $logger_additional_options[$option_name] : '';

						$logger->set_option($option_name, $option_value);
					}
				}

				// check if logger is active.
				$active = isset($options['eum_logger_type'][$logger_id]) && "1" === $options['eum_logger_type'][$logger_id] ? true : false;

				if ($active) {
					$logger->enable();
				} else {
					$logger->disable();
				}

				$loggers[] = $logger;
			}
		}

		$loggers = apply_filters('eum_loggers', $loggers);

		return $loggers;
	}

	/**
	 * Returns associative array with logger class name in a key and path to class file in a value.
	 *
	 * @return array
	 */
	public function get_loggers_classes() {
		$loggers_classes = array(
			'MPSUM_PHP_Logger'      => EASY_UPDATES_MANAGER_MAIN_PATH . 'includes/MPSUM_PHP_Logger.php',
			'MPSUM_Email_Logger'    => EASY_UPDATES_MANAGER_MAIN_PATH . 'includes/MPSUM_Email_Logger.php',
			'MPSUM_Syslog_Logger'   => EASY_UPDATES_MANAGER_MAIN_PATH .'includes/MPSUM_Syslog_Logger.php',
			'MPSUM_Slack_Logger'    => EASY_UPDATES_MANAGER_MAIN_PATH .'includes/MPSUM_Slack_Logger.php',
		);

		return apply_filters('eum_loggers_classes', $loggers_classes);
	}

	/**
	 * Displays settings on logs tab
	 */
	public function display_settings() {
		$loggers = $this->get_logger()->get_loggers();
		Easy_Updates_Manager()->include_template('external-logs.php', false, array('loggers' => $loggers));
	}

	/**
	 * Log automatic updates
	 *
	 * @since 6.0.0
	 * @access public
	 * @param array $update_results Update results
	 * @return void
	 */
	public function automatic_updates($update_results) {
		if (empty($update_results)) return;
		$messages = array();
		$this->log_messages = $this->get_cached_version_information();
		foreach ($update_results as $type => $results) {
			switch ($type) {
				case 'core':
					$core = $results[0];
					$name = $core->name;
					$status = 0;
					$version_from = $this->log_messages['core']['version'];
					$version = $this->log_messages['core']['new_version'];
					list($version, $status) = $this->set_status_and_version($core->result, $version_from, $version, $status);
					$messages[] = $this->get_message($name, $type, $version_from, $version, 'automatically', $status);
					break;
				case 'plugin':
					foreach ($results as $plugin) {
						if (!isset($plugin->item->plugin)) continue;
						$name = (isset($plugin->name ) && !empty($plugin->name)) ? $plugin->name : $plugin->item->slug;
						$status  = 0;
						$version = isset($plugin->item->new_version ) ? $plugin->item->new_version : '0.00';
						$version_from = $this->log_messages[$type][$plugin->item->plugin]['version'];
						list($version, $status) = $this->set_status_and_version($plugin->result, $version_from, $version, $status);
						$messages[] = $this->get_message($name, $type, $version_from, $version, 'automatically', $status);
					}
					break;
				case 'theme':
					foreach ($results as $theme) {
						if (!isset($theme->item->theme)) continue;
						$name = $theme->name;
						$status = 0;
						$version = $theme->item->new_version;
						$version_from = $this->log_messages[$type][$theme->item->theme]['version'];
						list($version, $status) = $this->set_status_and_version($theme->result, $version_from, $version, $status);
						$messages[] = $this->get_message($name, $type, $version_from, $version, 'automatically', $status);
					}
					break;
				case 'translation':
					foreach ($results as $translation) {
						$status = is_wp_error($translation->result) ? 0 : 1;
						$version_from = $translation->item->version;
						$version = ( 1 == $status ) ? $translation->item->version : '';
						$name = $this->get_name_for_update($translation->item->type, $translation->item->slug);
						$name = $name . ' (' . $translation->item->language . ')';
						$messages[] = $this->get_message($name, $type, $version_from, $version, 'automatically', $status);
					}
					break;
			}
		}
		$this->log($messages);
	}

	/**
	 * Manual updates
	 *
	 * @param array $upgrader_object Upgrade Object(s)
	 * @param array $options 		 Update options
	 * @return void
	 */
	public function manual_updates($upgrader_object, $options) {
		if (!isset($options['action']) || 'update' !== $options['action']) return;
		$user_id = get_current_user_id();
		if (0 == $user_id) return; // If there is no user, this is not a manual update
		if (true === $this->auto_update) return;
		$messages = array();
		$this->log_messages = $this->get_cached_version_information();
		switch ($options['type']) {
			case 'core':
				$version_from = $this->log_messages['core']['version'];
				$version = $this->log_messages['core']['new_version'];
				if (!empty($this->log_messages['core']['reinstall'])) {
					$status = 1;
				} else {
					$status = $version_from !== $version ? 1 : 0;
				}
				$name = 'WordPress ' . $version;
				$messages[] = $this->get_message($name, $options['type'], $version_from, $version, 'manually', $status, $user_id);
				break;
			case 'plugin':
				if (!empty($this->log_messages['plugin']) && isset($options['plugins']) && !empty($options['plugins'])) {
					foreach ($options['plugins'] as $plugin) {
						if (isset($this->log_messages['plugin'][$plugin])) {
							$version_from = $this->log_messages['plugin'][$plugin]['version'];
							$version = $this->log_messages['plugin'][$plugin]['new_version'];
							$status = ($version_from == $version) ? 0 : 1;
							$name = $this->log_messages['plugin'][$plugin]['name'];
							$messages[] = $this->get_message($name, $options['type'], $version_from, $version, 'manually', $status, $user_id);
						}
					}
				}
				break;
			case 'theme':
				if (isset($options['themes']) && !empty($options['themes'])) {
					foreach ($options['themes'] as $theme) {
						$theme_data = wp_get_theme($theme);
						if ($theme_data->exists()) {
							if (!empty($this->log_messages['theme'][$theme])) {
								$version_from = $this->log_messages['theme'][$theme]['version'];
								$version = $theme_data->get('Version');
								$status = ($version_from == $version) ? 0 : 1;
								$name = $theme_data->get('Name');
								$messages[] = $this->get_message($name, $options['type'], $version_from, $version, 'manually', $status, $user_id);
							}
						}
					}
				}
				break;
			case 'translation':
				foreach ($options['translations'] as $translation) {
					$status = 1;
					$version = $translation['version'];
					$version_from = $version;
					$slug = $translation['slug'];
					$name = $this->get_name_for_update($translation['type'], $slug);
					$name = $name . ' (' . $translation['language'] . ')';
					$messages[] = $this->get_message($name, $options['type'], $version_from, $version, 'manually', $status, $user_id);
				}
				break;
		}
		$this->log($messages);
	}

	/**
	 * Inserts result of upgrade process message to log table
	 *
	 * @param array $messages Upgrade name
	 */
	private function log($messages) {
		$loggers = $this->get_logger()->get_loggers();
		if (!empty($loggers)) {
			foreach ($loggers as $logger) {
				if (is_a($logger, 'MPSUM_Email_Logger')) {
					$logger->flush_log($messages);
				} else {
					foreach ($messages as $message) {
						$logger->info($message);
					}
				}
			}
		}
	}

	/**
	 * Creates a readable log message
	 *
	 * @param string $name         Upgrade name
	 * @param string $type         Type of upgrade
	 * @param string $version_from Upgrade from version number
	 * @param string $version      Upgrade to version number
	 * @param string $action       Action type, manual or automatic
	 * @param int    $status       Status of upgrade
	 * @param int    $user_id      ID of user who is responsible for update
	 *
	 * @return string A log message
	 */
	private function get_message($name, $type, $version_from, $version, $action, $status, $user_id = 0) {
		$url = is_multisite() ? network_site_url() : site_url();
		$type = ucfirst($type);
		$status_message = (1 === $status) ? __('Update is successful!', 'stops-core-theme-and-plugin-updates') : __('Update failed!', 'stops-core-theme-and-plugin-updates');
		if (0 === $user_id) {
			return sprintf('%1$s: %2$s: %3$s is updated from %4$s to %5$s %6$s at %7$s. %8$s', $url, $type, $name, $version_from, $version, $action, current_time('mysql'), $status_message);
		}
		$user = get_user_by('ID', $user_id);
		return sprintf('%1$s: %2$s: %3$s is updated from %4$s to %5$s %6$s at %7$s by user %8$s.', $url, $type, $name, $version_from, $version, $action, current_time('mysql'), $status_message, $user->user_login);
	}
}
